%%
% BacStalk
%
% Copyright (c) 2018 Raimo Hartmann & Muriel van Teeseling <bacstalk@gmail.com>
% Copyright (c) 2018 Drescher-lab, Max Planck Institute for Terrestrial Microbiology, Marburg, Germany
% Copyright (c) 2018 Thanbichler-lab, Philipps Universitaet, Marburg, Germany
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%%


function measurementFields = returnMeasurementFields(data, mode)

for i = 1:numel(data.frames)
    if ~isempty(data.frames(i).cells)
        measurementFields = fieldnames(data.frames(i).cells.Stats);
        break;
    end
end

nonValidFields = cellfun(@(x) ~isempty(x), strfind(lower(measurementFields), 'idx'))...
    | cellfun(@(x) ~isempty(x), strfind(lower(measurementFields), 'coordinates'))...
    | cellfun(@(x) ~isempty(x), strfind(lower(measurementFields), 'centroid'))...
    | cellfun(@(x) ~isempty(x), strfind(lower(measurementFields), 'medialaxisintensity'))...
    | cellfun(@(x) ~isempty(x), strfind(lower(measurementFields), 'stalkintensity'))...
    | cellfun(@(x) ~isempty(x), strfind(lower(measurementFields), 'cellid'));
measurementFields(nonValidFields) = [];

if strcmp(mode, 'all')
    % Add custom fields to table
    measurementFields = union(measurementFields, {'CellStalkLength', 'CellStalkBudLength', 'StalkBudLength', 'BudLength', 'BudArea'});
end

% Add filenames to cell table
for ch = 1:numel(data.settings.channelNames)
    measurementFields = union(measurementFields, sprintf('Filename_%s', data.settings.channelNames{ch}));
end

% Sort measurements
measurementFields = sort(measurementFields);

if strcmp(mode, 'all') && isfield(data.settings, 'customMeasurementFieldNames')
    measurementFields = [measurementFields; {data.settings.customMeasurementFieldNames.fieldName}'];
end